<?php
//Attacks.inc
//Defines the different kinds of attacks being checked for

/***********	Attack Definitions	***********************************/

interface IAttack
{
	public function GetInformationURL();
	public function GetAttackName();
	public function CheckForAttack($data);
	public function GetInfringing($data);
	public function IsWhitelisted($uri, $name);
}

class e107_syle_attacks implements IAttack
{
	public function GetInformationURL()
	{
		return "http://www.ticktockcomputers.com/wordpress/potential-for-php-code-execution-in-wordpress-themes-and-plug-ins/";
	}
	public function GetAttackName()
	{
		return "Potential [PHP] code execution attacks"; /* Based on e107 CMS, 3rd party plug-ins and themes may introduce this type of flaw */
	}
	public function CheckForAttack($data)
	{
		$data = urldecode($data);
			return ((substr_count($data,'[php') > 0) || (substr_count($data,'%5Bphp') > 0));
	}
	public function GetInfringing($data)
	{	
		if(preg_match('/:\/\/[^\/]+\/(.*)/', $data, $matches) == 1)
		{
			return $matches[1];
			//Lets see how this works -bwall
		}
		return strstr($data, substr($data, strpos($data, '/', 2)));
		//strstr($data, substr($data, strpos($data, '/', 2))); //count to 3rd / as in http://site.com/ then rest is attempts to bruteforce or retrieve usernames on the system
		//Need bwall to rewrite this string data since its not showing on the log.
	}
	
	public function IsWhitelisted($uri, $name)
	{
		return false;
	}
}

class UsernameDeduction implements IAttack
{
	public function GetInformationURL()
	{
		return "http://www.ticktockcomputers.com/humor/wordpress-honeypots-via-wp-includescomment-template-php/";
	}		
	public function GetAttackName()
	{
		return "WordPress Username Deduction";
	}		
	public function CheckForAttack($data)
	{
		return ((substr_count($data, '/author/') > 0) || (substr_count($data, '?author=') > 0));
	}							
	public function GetInfringing($data)
	{	
		if(preg_match('/:\/\/[^\/]+\/(.*)/', $data, $matches) == 1)
		{
			return $matches[1];
			//Lets see how this works -bwall
		}
		return strstr($data, substr($data, strpos($data, '/', 2)));
		//strstr($data, substr($data, strpos($data, '/', 2))); //count to 3rd / as in http://site.com/ then rest is attempts to bruteforce or retrieve usernames on the system
		//Need bwall to rewrite this string data since its not showing on the log.
	}
	public function IsWhitelisted($uri, $name)
	{
		return false;
	}
}

class DirectoryTraversal implements IAttack
{
	public function GetInformationURL()
	{
		return "http://en.wikipedia.org/wiki/Directory_traversal";
	}		
	public function GetAttackName()
	{
		return "Directory Traversal/Local File Inclusion";
	}		
	public function CheckForAttack($data)
	{
		return ((substr_count($data, '../') > 0) || (substr_count($data, 'file=') > 0));
	}
	public function GetInfringing($data)
	{
		return strstr($data, '?');
	}
	public function IsWhitelisted($uri, $name)
	{
		if(substr_count($uri, "wp-comments-post.php") > 0)
		{
			if($name == "comment")
			{
				return true;
			}	
		}
		return false;
	}
}

class WordPressTrackBack implements IAttack
{
	public function GetInformationURL()
	{
		return "http://en.wikipedia.org/wiki/Trackback";
	}
	public function GetAttackName()
	{
		return "WordPress Trackback";
	}
	public function CheckForAttack($data)
	{
		$data = urldecode($data);
		return (substr_count($data,'://') > 0);
	}
	public function GetInfringing($data)
	{
		return strstr($data, '?');
	}
	public function IsWhitelisted($uri, $name)
	{
		$whitelisted = true;
		if(!(substr_count($uri, "wp-comments-post.php") > 0))
		{
			if($name == "url")
			{
				$whitelisted = false;
			}
		}
		return $whitelisted;
	}
}

class RemoteFileInclusion implements IAttack
{
	public function GetInformationURL()
	{
		return "http://en.wikipedia.org/wiki/Remote_File_Inclusion";
	}
	public function GetAttackName()
	{
		return "Remote File Inclusion";
	}
	public function CheckForAttack($data)
	{
		$data = strtolower(urldecode($data));
		return (preg_match('/(http|https|ftp):\/\/[^\s]+/', $data) > 0);
	}
	public function GetInfringing($data)
	{		
		return strstr($data, '?');		
	}
	public function IsWhitelisted($uri, $name)
	{
		$whitelisted = false;
		if($name == "url")
		{
			$whitelisted = true;
		}
		if(substr_count($uri, "wp-comments-post.php") > 0)
		{
			if($name == "url" || $name == "comment")
			{
				$whitelisted = true;
			}	
		}
		if(substr_count($uri, "wp-login.php") > 0)
		{
			if($name == "redirect_to")
			{
				$whitelisted = true;
			}
		}
		return $whitelisted;
	}
}


class XSS implements IAttack
{
	public function GetInformationURL()
	{
		return "http://en.wikipedia.org/wiki/Cross_site_scripting";
	}
	public function GetAttackName()
	{
		return "XSS";
	}
	public function CheckForAttack($data)
	{
		$data = urldecode(strtolower($data));
		return ((substr_count($data,'</script') > 0) || (substr_count($data,'<script') > 0)|| 
		(substr_count($data,'javascript:') > 0) || (substr_count($data,'\0') > 0)|| (substr_count($data,'%00') > 0));
	}
	public function GetInfringing($data)
	{
		return strstr($data, '?');
	}
	public function IsWhitelisted($uri, $name)
	{
		if(substr_count($uri, "wp-comments-post.php") > 0)
		{
			if($name == "comment")
			{
				return true;
			}	
		}
		return false;
	}	
}

class SQLi implements IAttack
{
	public function GetInformationURL()
	{
		return "http://en.wikipedia.org/wiki/SQL_injection";
	}
	public function GetAttackName()
	{
		return "SQLi";
	}
	public function CheckForAttack($data)
	{
		$data = strtolower(urldecode($data));
		return ((substr_count($data,'order by ') > 0) || 
				(substr_count($data,'or \'') > 0) || 
				(substr_count($data,' or 1') > 0) || (substr_count($data,'UNION ALL') > 0) ||(substr_count($data,'SELECT * FROM') > 0) );
	}
	public function GetInfringing($data)
	{
		return strstr($data, '?');
	}
	public function IsWhitelisted($uri, $name)
	{
		if(substr_count($uri, "wp-comments-post.php") > 0)
		{
			if($name == "comment")
			{
				return true;
			}	
		}
		return false;
	}
}

class SourceDisclosure implements IAttack
{
	public function GetInformationURL()
	{
		return "https://bugs.php.net/bug.php?id=61910";
	}
	public function GetAttackName()
	{
		return "PHP/CGI Source Code Disclosure/Coded Execution";
	}
	public function CheckForAttack($data)
	{
		return ((substr_count($data,'?-s') > 0) || 
				(substr_count($data,'?-d') > 0));
	}
	public function GetInfringing($data)
	{
		return strstr($data, '?');
	}
	public function IsWhitelisted($uri, $name)
	{
		if(substr_count($uri, "wp-comments-post.php") > 0)
		{
			if($name == "comment")
			{
				return true;
			}	
		}
		return false;
	}
}

class ListAllApache implements IAttack
{
	public function GetInformationURL()
	{
		return "http://www.slideshare.net/rob.ragan/filter-evasion-houdini-on-the-wire";
	}
	public function GetAttackName()
	{
		return "Filter Evasion";
	}
	public function CheckForAttack($data)
	{
		return (substr_count($data,'/%3f') > 0);
	}
	public function GetInfringing($data)
	{
		return strstr($data, '/%3f');
	}
	public function IsWhitelisted($uri, $name)
	{
		if(substr_count($uri, "wp-comments-post.php") > 0)
		{
			if($name == "comment")
			{
				return true;
			}	
		}
		return false;
	}
}

?>
