<?php

function xssafe($data,$encoding='UTF-8')
{
	return htmlspecialchars($data,ENT_QUOTES | ENT_HTML401,$encoding);
}

//Logging Classes
$ipCache = array();

class IPInformation
{
	public $data = array();
	public $ip;

	public function GetRemoteInfo($ip)
	{		
		$this->ip = $ip;
		$postdata = http_build_query(array('format' => "json", 'ip' => $ip));
		$opts = array(
		  'http'=>array(
			'method'=>'POST',
			'header'=>'Content-type: application/x-www-form-urlencoded',
			'content' => $postdata,
			'protocol_version' => 1.1
		  )
		);
		$context = stream_context_create($opts);
		$infosniper = file_get_contents('http://www.attack-scanner.com/geoip/geo-ip-test.php', false, $context);
		$this->data = json_decode($infosniper);
	}
	
	public function PrintIPInformation()
	{
		$html = "<div class=\"info-table\">Attacker Location Info:<br /><table cellspacing=\"0\"  class=\"info-sniper\">
		<tr>
			<th class=\"content-td1\">IP Address</th>
			<th width=\"20\">&nbsp;</td>
			<th class=\"content-td1\">City</th>
			<th width=\"20\">&nbsp;</th>
			<th class=\"content-td1\">Country</th>
			<th width=\"20\">&nbsp;</th>
			<th class=\"content-td1\">Region</th>
		</tr>
		<tr>
			<td class=\"content-td2\">".xssafe($this->data->IP)."</td>
			<td width=\"20\">&nbsp;</td>
			<td class=\"content-td2\">".xssafe($this->data->City)."</td>
			<td width=\"20\">&nbsp;</td>
			<td class=\"content-td2\">".xssafe($this->data->CountryName)."</td>
			<td width=\"20\">&nbsp;</td>
			<td class=\"content-td2\">".xssafe($this->data->Region)."</td>
		</tr>
		<tr>
			<th class=\"content-td1\">Postal Code</th>
			<th width=\"20\">&nbsp;</th>
			<th class=\"content-td1\">Latitude</th>
			<th width=\"20\">&nbsp;</th>
			<th class=\"content-td1\">Longitude</th>
			<th width=\"20\">&nbsp;</th>
			<th class=\"content-td1\">Hostname</th>
		</tr>
		<tr>
			<td class=\"content-td2\">".xssafe($this->data->PostalCode)."</td>
			<td width=\"20\">&nbsp;</td>
			<td class=\"content-td2\">".xssafe($this->data->Latitude)."</td>
			<td width=\"20\">&nbsp;</td>
			<td class=\"content-td2\">".xssafe($this->data->Longitude)."</td>
			<td width=\"20\">&nbsp;</td>
			<td class=\"content-td2\">".xssafe($this->data->Hostname)."</td>
		</tr>
	  </table><br /><br />
	  
	  </div>";
		return $html;
	}
}

class LogEntry
{
	public $ip;
	public $attack;
	public $date;
	public $hostaddress;
	public $browser;
	public $referred;
	public $method;
	public $text;
	public $rfi;
	public $ipInformation;

	public function __construct($ip, $attack, $date, $hostaddress, $browser, $referred, $method, $text, $rfi)
	{
		$this->ip = $ip;
		if(isset($ipCache[$ip]))
		{
			$this->ipInformation = $ipCache[$ip];
		}
		else
		{
			$this->ipInformation = new IPInformation();
			$this->ipInformation->GetRemoteInfo($ip);
			$ipCache[$ip] = $this->ipInformation;
		}		
		$this->attack = xssafe($attack);
		$this->date = xssafe($date);
		$this->hostaddress = xssafe($hostaddress);
		$this->browser = xssafe($browser);
		$this->referred = xssafe($referred);
		$this->method = xssafe($method);
		$this->text = xssafe($text);
		$this->rfi = xssafe($rfi);
	}
}

function LineToLogEntry($buffer)
{
	$encPass = get_option("AttackScannerEncPass");
	if($encPass === false)
	{
		add_option("AttackScannerEncPass", "Change Me", "", "yes");
		$encPass = get_option("AttackScannerEncPass");
	}
	$crypt = new ARCbwall($encPass);
	$arr = unserialize(gzinflate($crypt->Crypt(base64_decode($buffer))));
	return $arr;
}

function GetAllLogEntries($includeArchive = false)
{
	$entries = array();
	if($includeArchive === true)
	{
		$myFile = dirname(__FILE__)."/archive.txt";
		$handle = fopen($myFile, "r");
		if ($handle) 
		{		
			$lines = array();
			$index = 1;
			while (($buffer = fgets($handle)) !== false) 
			{
				$buffer = trim($buffer);
				if($buffer != "")
				{
					$arr = LineToLogEntry($buffer);
					array_push($entries, $arr);
					$index++;
				}
			}
			fclose($handle);
		}
	}
	
	$myFile = dirname(__FILE__)."/data.txt";
	$handle = fopen($myFile, "r");
	if ($handle) 
	{		
		$lines = array();
		$index = 1;
		while (($buffer = fgets($handle)) !== false) 
		{
			$buffer = trim($buffer);
			if($buffer != "")
			{
				$arr = LineToLogEntry($buffer);
				array_push($entries, $arr);
				$index++;
			}
		}
		fclose($handle);
	}
	return $entries;
}

function EntryToCSV($entry)
{
	return "\"".$entry->date."\",\"".$entry->attack."\",\"".$entry->ip."\",\"".$entry->hostaddress."\",\"".$entry->rfi."\",\"".$entry->text."\",\"".$entry->method."\",\"".$entry->browser."\",\"".$entry->referred."\",\"".$entry->ipInformation->data->Latitude."\",\"".$entry->ipInformation->data->Longitude."\"";
}

function AddLogEntry($log)
{
	$encPass = get_option("AttackScannerEncPass");
	if($encPass === false)
	{
		add_option("AttackScannerEncPass", "Change Me", "", "yes");
		$encPass = get_option("AttackScannerEncPass");
	}
	$crypt = new ARCbwall($encPass);
	file_put_contents(dirname(__FILE__).'/data.txt', base64_encode($crypt->Crypt(gzdeflate(serialize($log), 9)))."\n", FILE_APPEND | LOCK_EX);
	//Add calls to modules here later
	
	//Send LogEntry to central logging server
	$json = json_encode($log);
	$postdata = http_build_query(array('entry' => urlencode(base64_encode($json))));
	$opts = array(
	  'http'=>array(
		'method'=>'POST',
		'header'=>'Content-type: application/x-www-form-urlencoded',
		'content' => $postdata,
		'protocol_version' => 1.1
	  )
	);
	$context = stream_context_create($opts);
	file_get_contents('http://www.attack-scanner.com/central-logging/submitlog.php', false, $context); //Mandatory in Free version
	
	if($log->attack == "Remote File Inclusion")
	{
		if(true)
		{
			if(preg_match('/.*(?<url>https?:\/\/[^<>[:space:]]+)/', urldecode($log->rfi), $matches) > 0)
			{
				$url = $matches['url'];
				$postdata = http_build_query(
					array('u' => urlencode(base64_encode($url)))
				);
				$opts = array(
				  'http'=>array(
					'method'=>'POST',
					'header'=>'Content-type: application/x-www-form-urlencoded',
					'content' => $postdata,
					'protocol_version' => 1.1
				  )
				);
				$context = stream_context_create($opts);
				file_get_contents('https://firebwall.com/decoding/submit.php', false, $context);
			}
		}
	}
}

function TrimLog($maxLogEntries = 100, $archiving = true)
{
	$myFile = dirname(__FILE__)."/data.txt";
	$handle = fopen($myFile, "r");
	if ($handle) 
	{		
		$lines = array();
		$index = 1;
		while (($buffer = fgets($handle)) !== false) 
		{
			$buffer = trim($buffer);
			if($buffer != "")
			{
				array_push($lines, $buffer);
				$index++;
			}
		}
		fclose($handle);
		if($index > $maxLogEntries)
		{
			//log needs to be trimmed
			$output = "";
			$archive = "";
			for($i = 0; $i < $index - 1; $i++)
			{
				if($index - $i <= $maxLogEntries + 1)
				{
					$output .= $lines[$i]."\n";
				}
				else
				{
					$archive .= $lines[$i]."\n";
				}
			}
			file_put_contents(dirname(__FILE__)."/data.txt", $output, LOCK_EX);
			if($archiving)
			{
				file_put_contents(dirname(__FILE__)."/archive.txt", $archive, LOCK_EX | FILE_APPEND);
			}
		}
	}
}

?>
